class Reliability{

	alpha(itemScoresMatrix, scoresArray){//crohnbach's alpha; item scores in 2-dimensional array
		let stat = new Stat();
		let numberOfItems = itemScoresMatrix.length;
		let variance = stat.variance(scoresArray);
		let sumOfItemVariances = 0;
		let factor1;
		let factor2;
		let i;
		for(i = 0; i < numberOfItems; i++)
			sumOfItemVariances += stat.variance(itemScoresMatrix[i]);
		factor1 = numberOfItems / (numberOfItems - 1);
		factor2 = sumOfItemVariances / variance;
		return factor1 * (1 - factor2);
	}
	
	alphaStrat(varArray, relArray, totalVariance){//stratified alpha coefficient from subtest variances, subtest reliabilities, and total variance
		let errorSum = 0;
		let i;
		for(i = 0; i < varArray.length; i++)
			errorSum += varArray[i] * (1 - relArray[i]);
		return 1 - (errorSum / totalVariance);
	}

	divBy(n, by){
		return n % by === 0 ? true : false;
	}
	
	error(sdev, rel){//standard error of measurement
		return sdev * Math.sqrt(1 - rel);
	}

	halfScores(itemScoresMatrix, zeroOrOne){
	 let numberOfItems = itemScoresMatrix.length;
	 let n = itemScoresMatrix[0].length;
	 let halfs = [];
	 let i, j;
	 for(i = 0; i < n; i++)
	  halfs[i] = 0;
	 for(j = zeroOrOne; j < numberOfItems; j += 2)
	  for(i = 0; i < n; i++)
	   halfs[i] += itemScoresMatrix[j][i];
	 return halfs;   
	}

	halfScores34(itemScoresMatrix){
	 let numberOfItems = itemScoresMatrix.length;
	 let n = itemScoresMatrix[0].length;
	 let halfs = [];
	 let i, j;
	 for(i = 0; i < n; i++)
	  halfs[i] = 0;
	 for(j = 3; j < numberOfItems; j++)
	  if(this.divBy(j, 3) || this.divBy(j, 4))
	   for(i = 0; i < n; i++)
		halfs[i] += itemScoresMatrix[j][i];
	 return halfs;   
	}

	halfScoresRest(itemScoresMatrix){
	 let numberOfItems = itemScoresMatrix.length;
	 let n = itemScoresMatrix[0].length;
	 let halfs = [];
	 let i, j;
	 for(i = 0; i < n; i++)
	  halfs[i] = 0;
	 for(j = 0; j < numberOfItems; j++)
	  if(!this.divBy(j, 3) && !this.divBy(j, 4))
	   for(i = 0; i < n; i++)
		halfs[i] += itemScoresMatrix[j][i];
	 return halfs;   
	}

	spearmanBrown(oddEven){
	 return 2 * oddEven / (1 + oddEven);
	}
	
	spearmanBrownExpanded(relArray, varArray){//to compute compound reliability from subtest reliabilities and subtest variances
		let upper = 0;
		let lower = 0;
		let i;
		for(i = 0; i < relArray.length; i++){
			upper += Math.sqrt(varArray[i]) * 2 * relArray[i];
			lower += Math.sqrt(varArray[i]) * (1 + relArray[i]);
		}
		return upper / lower;
	}

	splitHalf(itemScoresMatrix){//split-half reliability odd-even
	 let stat = new Stat();
	 let oddScore = this.halfScores(itemScoresMatrix,0);
	 let evenScore = this.halfScores(itemScoresMatrix,1);
	 let cor = stat.cor(oddScore, evenScore);
	 return this.spearmanBrown(cor);
	}

	splitHalf34(itemScoresMatrix){//split-half reliability other division
	 let stat = new Stat();
	 let scores34 = this.halfScores34(itemScoresMatrix);
	 let scoresRest = this.halfScoresRest(itemScoresMatrix);
	 let cor = stat.cor(scores34, scoresRest);
	 return this.spearmanBrown(cor);
	}
}
//This code can be studied, used, distributed, and adapted freely by anyone
